'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolveFetchStatus = exports.resolveInitialFetchStatus = exports.Provider = exports.FetchStatus = exports.connectAuth = exports.connect = undefined;
exports.default = init;

var _batch = require('./middleware/batch');

var _connect = require('./connect');

var _connect2 = _interopRequireDefault(_connect);

var _connectAuth = require('./connectAuth');

var _connectAuth2 = _interopRequireDefault(_connectAuth);

var _app = require('@firebase/app');

var _app2 = _interopRequireDefault(_app);

require('@firebase/auth');

require('@firebase/firestore');

var _fetchStatus = require('./modules/fetchStatus');

var _Provider = require('./Provider');

var _Provider2 = _interopRequireDefault(_Provider);

var _reducers = require('./reducers');

var _reducers2 = _interopRequireDefault(_reducers);

var _selectors = require('./selectors');

var _actions = require('./actions');

var _reduxThunk = require('redux-thunk');

var _reduxThunk2 = _interopRequireDefault(_reduxThunk);

var _redux = require('redux');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var store = void 0;
function init(app, userCollection) {
  if (store) {
    throw new Error('Cannot initialize store more than once.');
  }

  var firestore = _app2.default.firestore(app);
  var auth = _app2.default.auth(app);

  var thunkArgs = { auth: auth, firestore: firestore };
  var middleware = [_reduxThunk2.default.withExtraArgument(thunkArgs), _batch.batchMiddleware];

  if (process.env.NODE_ENV !== 'production') {
    var createLogger = require('redux-logger').createLogger;
    var logger = createLogger({ collapsed: true, diff: true });
    middleware.push(logger);
  }

  store = (0, _redux.createStore)(_reducers2.default, _redux.applyMiddleware.apply(undefined, middleware));
  if (process.env.NODE_ENV !== 'production') {
    window.redux = store;
  }

  var select = (0, _selectors.initSelect)(store);
  var selectAuth = (0, _selectors.initSelectAuth)(auth, userCollection);
  var selectStorage = (0, _selectors.initSelectStorage)(store);

  var currentUser = auth.currentUser;
  var currentUid = void 0;
  if (currentUser) {
    currentUid = currentUser.uid;
    store.dispatch((0, _actions.setUser)(currentUser, userCollection));
  }

  return new Promise(function (resolve, reject) {
    auth.onAuthStateChanged(function (newUser) {
      store.dispatch((0, _actions.setUser)(newUser, userCollection)).then(function () {
        resolve({ app: app, select: select, selectAuth: selectAuth, selectStorage: selectStorage, store: store });
      }).catch(function (error) {
        reject(error);
      });
      if (newUser) {
        currentUid = newUser.uid;
      } else {
        store.dispatch((0, _actions.unsetUser)(currentUid, userCollection));
      }
    }, function (error) {
      reject(error);
    });
  });
}

exports.connect = _connect2.default;
exports.connectAuth = _connectAuth2.default;
exports.FetchStatus = _fetchStatus.FetchStatus;
exports.Provider = _Provider2.default;
exports.resolveInitialFetchStatus = _fetchStatus.resolveInitialFetchStatus;
exports.resolveFetchStatus = _fetchStatus.resolveFetchStatus;
//# sourceMappingURL=index.js.map