'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var BATCH_ACTION = 'BATCH';

var isPlainAction = function isPlainAction(action) {
  return (typeof action === 'undefined' ? 'undefined' : _typeof(action)) === 'object' && typeof action.type === 'string';
};

var actionBatch = function actionBatch(actions) {
  var meta = actions.reduce(function (obj, action) {
    if (_typeof(action.meta) === 'object') {
      Object.assign(obj, action.meta);
    }
    return obj;
  }, {});
  return {
    type: BATCH_ACTION,
    payload: actions,
    meta: meta
  };
};

var batchMiddleware = exports.batchMiddleware = function batchMiddleware(_ref) {
  var dispatch = _ref.dispatch;
  return function (next) {
    return function (actions) {
      if (Array.isArray(actions)) {
        var plainActions = [];
        actions.forEach(function (action) {
          if (typeof action === 'function') {
            dispatch(action);
          } else if (isPlainAction(action)) {
            plainActions.push(action);
          }
        });
        return plainActions.length ? next(actionBatch(plainActions)) : undefined;
      } else {
        return next(actions);
      }
    };
  };
};

var batchReducer = exports.batchReducer = function batchReducer(reducer) {
  return function (state, action) {
    return action && action.type === BATCH_ACTION ? action.payload.reduce(reducer, state) : reducer(state, action);
  };
};
//# sourceMappingURL=batch.js.map