'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var emptyArray = [];

// $FlowFixMe Undocumented key/value

var pathToString = function pathToString(path) {
  return path.segments.join('/');
};

var filterToString = function filterToString(filter) {
  var field = filter.field,
      op = filter.op,
      value = filter.value;

  return '' + pathToString(field) + op.name + value.toString();
};

var getQueryId = exports.getQueryId = function getQueryId(query) {
  // $FlowFixMe Undocumented key/value
  if (query.id && query.id === query.path) {
    // $FlowFixMe Undocumented key/value
    return query.id;
  }

  if (query.path && typeof query.path === 'string') {
    // $FlowFixMe Undocumented key/value
    return query.path;
  }

  if (query._query) {
    // $FlowFixMe Grabbing private value
    var _query$_query = query._query,
        filters = _query$_query.filters,
        limit = _query$_query.limit,
        path = _query$_query.path;

    var filterString = filters.map(filterToString).join('|');
    return '' + pathToString(path) + (filterString ? ':' + filterString : '') + (limit ? ':' + limit : '');
  }

  throw new Error('Unknown query type');
};

var getQueryPath = exports.getQueryPath = function getQueryPath(query) {
  if (query.path) {
    // $FlowFixMe Undocumented key/value
    return query.path;
  }

  if (query._query) {
    // $FlowFixMe Grabbing private value
    var path = query._query.path;

    return pathToString(path);
  }

  throw new Error('Unknown query type');
};

var getCollectionQueryPath = exports.getCollectionQueryPath = function getCollectionQueryPath(query) {
  var fullPath = getQueryPath(query);
  var pathParts = fullPath.split('/');
  return pathParts.length % 2 ? fullPath : fullPath.replace(/(\/[^/]+)$/, '');
};

var _getValueFromFieldPath = function _getValueFromFieldPath(doc, fieldPath) {
  var value = doc;
  // $FlowFixMe Undocumented key/value
  for (var i = 0; i < fieldPath.segments.length; i++) {
    // $FlowFixMe Undocumented key/value
    var segment = fieldPath.segments[i];
    value = value && (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object' ? value[segment] : undefined;
    if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === undefined) {
      return undefined;
    }
  }
  return value;
};

var getDocumentIdsForQuery = exports.getDocumentIdsForQuery = function getDocumentIdsForQuery(query, state) {
  var collectionPath = getCollectionQueryPath(query);
  var queryPath = getQueryPath(query);
  var pathParts = queryPath.split('/');
  var documents = state.collections[collectionPath];
  if (!(pathParts.length % 2)) {
    var documentId = pathParts[pathParts.length - 1];
    return Object.keys(documents).indexOf(documentId) >= 0 ? [documentId] : [];
  }

  if (query._query) {
    // $FlowFixMe Undocumented key/value
    var _query$_query2 = query._query,
        filters = _query$_query2.filters,
        limit = _query$_query2.limit,
        orderBy = _query$_query2.orderBy;

    var docs = Object.values(documents).filter(function (doc) {
      return filters.every(function (_ref) {
        var field = _ref.field,
            op = _ref.op,
            value = _ref.value;

        var docValue = _getValueFromFieldPath(doc, field);
        var internalValue = value.internalValue;

        var normValue = (typeof internalValue === 'undefined' ? 'undefined' : _typeof(internalValue)) === 'object' ? internalValue.seconds ? new Date(internalValue.seconds) : internalValue : internalValue;
        switch (op.name) {
          case '<':
            return docValue < normValue;
          case '<=':
            return docValue <= normValue;
          case '==':
            return docValue == normValue;
          case '>':
            return docValue > normValue;
          case '>=':
            return docValue >= normValue;
          default:
            return true;
        }
      });
    });

    if (orderBy && orderBy.length > 0) {
      orderBy.forEach(function (order) {
        var dir = order.dir.name,
            field = order.field;

        docs = docs.sort(function (a, b) {
          var aValue = _getValueFromFieldPath(a, field);
          var bValue = _getValueFromFieldPath(b, field);
          if (!aValue || !bValue) {
            return 0;
          }
          if (typeof aValue === 'string' && typeof bValue === 'string') {
            return dir === 'asc' ? aValue.localeCompare(bValue) : bValue.localeCompare(aValue);
          }
          if (aValue instanceof Date && bValue instanceof Date) {
            return dir === 'asc' ? aValue.valueOf() - bValue.valueOf() : bValue.valueOf() - aValue.valueOf();
          }
          if (typeof aValue === 'number' && typeof bValue === 'number') {
            return dir === 'asc' ? aValue - bValue : bValue - aValue;
          }
          return 0;
        });
      });
    }

    return docs.map(function (doc) {
      return (typeof doc === 'undefined' ? 'undefined' : _typeof(doc)) === 'object' && doc && typeof doc.id === 'string' ? doc.id : undefined;
    }).filter(Boolean).slice(0, limit || Infinity);
  }
  return emptyArray;
};
//# sourceMappingURL=query.js.map