'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.reducer = reducer;

var _actions = require('../actions');

var _fetchStatus = require('../modules/fetchStatus');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var defaultState = {};
var defaultQueryState = {
  documentIds: [],
  error: undefined,
  fetchStatus: _fetchStatus.FetchStatus.NONE
};

var sortChanges = function sortChanges(a, b) {
  return a.doc.newIndex - b.doc.newIndex;
};

var updateMultiple = function updateMultiple(state, payload, queryId) {
  var documentIds = payload.docChanges.sort(sortChanges).reduce(function (memo, change) {
    var doc = change.doc,
        changeType = change.type,
        newIndex = change.newIndex;

    switch (changeType) {
      case 'added':
      case 'modified':
        if (memo.indexOf(doc.id) !== -1) {
          return memo;
        }
        if (memo.length === 0) {
          return [doc.id];
        }
        memo.splice(newIndex, 0, doc.id);
        return memo;
      case 'removed':
        memo.splice(newIndex, 1);
        return memo;
      default:
        return memo;
    }
  }, [].concat(_toConsumableArray(state[queryId].documentIds)));
  return _extends({}, state, _defineProperty({}, queryId, _extends({}, state[queryId], {
    documentIds: documentIds,
    fetchStatus: _fetchStatus.FetchStatus.LOADED
  })));
};

var updateOne = function updateOne(state, payload, queryId) {
  return _extends({}, state, _defineProperty({}, queryId, _extends({}, state[queryId], {
    documentIds: [payload.id],
    fetchStatus: _fetchStatus.FetchStatus.LOADED
  })));
};

function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultState;
  var action = arguments[1];

  switch (action.type) {
    case _actions.QUERIES.REQUEST:
      {
        var _queryId = action.meta.queryId;

        return _extends({}, state, _defineProperty({}, _queryId, _extends({}, defaultQueryState, state[_queryId], {
          fetchStatus: _fetchStatus.FetchStatus.LOADING
        })));
      }

    case _actions.QUERIES.SUCCESS:
      {
        var _queryId2 = action.meta.queryId,
            payload = action.payload;

        if (payload.docChanges) {
          return updateMultiple(state, payload, _queryId2);
        } else {
          return updateOne(state, payload, _queryId2);
        }
      }

    case _actions.QUERIES.FAILURE:
      {
        var _queryId3 = action.meta.queryId,
            _payload = action.payload;

        return _extends({}, state, _defineProperty({}, _queryId3, _extends({}, state[_queryId3], {
          error: _payload,
          fetchStatus: _fetchStatus.FetchStatus.FAILED
        })));
      }

    case _actions.QUERIES.ADD:
      {
        var _queryId4 = action.meta.queryId,
            _documentIds = action.payload;

        return _extends({}, state, _defineProperty({}, _queryId4, _extends({}, state[_queryId4], {
          documentIds: _documentIds,
          fetchStatus: _fetchStatus.FetchStatus.LOADED
        })));
      }

    case _actions.QUERIES.REMOVE:
      {
        var _queryId5 = action.meta.queryId;

        var newState = _extends({}, state);
        delete newState[_queryId5];
        return newState;
      }

    default:
      return state;
  }
}
//# sourceMappingURL=queries.js.map