'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initSelectStorage = exports.initSelectAuth = exports.initSelect = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _reselect = require('reselect');

var _fetchStatus = require('./modules/fetchStatus');

var _query = require('./modules/query');

var _actions = require('./actions');

var selectOptionDefaults = {
  subscribe: true
};

var stroageOptionDefaults = {
  metadata: false
};

var emptyArray = [];

var singleUndefined = { fetchStatus: _fetchStatus.FetchStatus.NONE, doc: undefined };
var collectionUndefiend = { fetchStatus: _fetchStatus.FetchStatus.NONE, docs: emptyArray };

var selectQueries = function selectQueries(state) {
  return state.queries;
};

var initSelect = exports.initSelect = function initSelect(store) {
  return function (query, selectOptions) {
    var options = _extends({}, selectOptionDefaults, selectOptions);
    var queryId = (0, _query.getQueryId)(query);
    var queryPath = (0, _query.getQueryPath)(query);
    var collectionQueryPath = (0, _query.getCollectionQueryPath)(query);
    var isDocument = collectionQueryPath !== queryPath;

    var selectStoreQuery = function selectStoreQuery(state) {
      return state.queries[queryId];
    };
    var selectCollection = function selectCollection(state) {
      return state.collections[collectionQueryPath];
    };

    var selector = (0, _reselect.createSelector)([selectStoreQuery, selectCollection], function (storeQuery, collection) {
      if (storeQuery && collection) {
        var documentIds = storeQuery.documentIds,
            fetchStatus = storeQuery.fetchStatus;

        if (documentIds) {
          var docs = documentIds.map(function (id) {
            return collection[id];
          }).filter(Boolean);
          return isDocument ? { fetchStatus: fetchStatus, doc: docs[0] } : { fetchStatus: fetchStatus, docs: docs };
        }
      }
      return isDocument ? singleUndefined : collectionUndefiend;
    });

    return function () {
      var actions = [(0, _actions.addQuery)(query)];
      if (options.subscribe) {
        actions.push((0, _actions.addListener)(query));
      }
      store.dispatch(actions);
      return selector;
    };
  };
};

var initSelectAuth = exports.initSelectAuth = function initSelectAuth(auth, userCollection) {
  var loggedOut = { fetchStatus: _fetchStatus.FetchStatus.LOADED, doc: undefined };
  var selectUid = function selectUid(state) {
    return state.auth.uid;
  };
  var selectStoreQuery = (0, _reselect.createSelector)([selectUid, selectQueries], function (uid, queries) {
    return userCollection && queries ? queries['auth|' + userCollection + '/' + uid] : undefined;
  });
  var selectUsersCollection = function selectUsersCollection(state) {
    return userCollection ? state.collections[userCollection] : undefined;
  };
  var selectUserData = (0, _reselect.createSelector)([selectUid, selectUsersCollection], function (uid, users) {
    return users ? users[uid] : undefined;
  });
  var selectCurrentUser = function selectCurrentUser() {
    return auth.currentUser ? JSON.stringify(auth.currentUser.toJSON()) : undefined;
  };

  var selector = (0, _reselect.createSelector)([selectUid, selectCurrentUser, selectStoreQuery, selectUserData], function (uid, currentUser, storeQuery, doc) {
    if (!uid) {
      return loggedOut;
    }
    var fetchStatus = storeQuery ? storeQuery.fetchStatus : !userCollection ? _fetchStatus.FetchStatus.LOADED : _fetchStatus.FetchStatus.NONE;
    return {
      fetchStatus: fetchStatus,
      doc: userCollection ? doc : {} // use a new object if no userCollection to cache bust
    };
  });

  return selector;
};

var initSelectStorage = exports.initSelectStorage = function initSelectStorage(store) {
  return function (ref, storageOptions) {
    var options = _extends({}, stroageOptionDefaults, storageOptions);
    var actionCreator = options.metadata ? _actions.getStorageMetadata : _actions.getStorageDownloadUrl;
    var selector = function selector(state) {
      return state.storage[ref.fullPath];
    };
    return function () {
      store.dispatch(actionCreator(ref));
      return selector;
    };
  };
};
//# sourceMappingURL=selectors.js.map